'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _storyboard = require('storyboard');

var _webpack = require('webpack');

var _webpack2 = _interopRequireDefault(_webpack);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var pkg = require('../../package.json');

var fProduction = process.env.NODE_ENV === 'production';

_storyboard.mainStory.info('webpack', 'Webpack configuration:', {
  attach: {
    environment: fProduction ? 'PRODUCTION' : 'DEVELOPMENT',
    version: pkg.version
  }
});

var _entry = function _entry(file) {
  return fProduction ? [file] : ['webpack-hot-middleware/client?reload=true', file];
};

var _styleLoader = function _styleLoader(loaderDesc) {
  return 'style!' + loaderDesc;
};

exports.default = {

  // -------------------------------------------------
  // Input (entry point)
  // -------------------------------------------------
  entry: { app: _entry('./src/client/startup.js') },

  // -------------------------------------------------
  // Output
  // -------------------------------------------------
  output: {
    filename: '[name].bundle.js',

    // Where PRODUCTION bundles will be stored
    path: _path2.default.resolve(process.cwd(), 'public/assets'),

    publicPath: '/assets/',

    libraryTarget: undefined
  },

  // -------------------------------------------------
  // Configuration
  // -------------------------------------------------
  devtool: fProduction ? undefined : 'eval',

  resolve: {
    // Add automatically the following extensions to required modules
    extensions: ['', '.jsx', '.js']
  },

  plugins: function () {
    var ret = [function pluginCompile() {
      this.plugin('compile', function () {
        return _storyboard.mainStory.debug('webpack', 'Bundling...');
      });
    }, function pluginDone() {
      this.plugin('done', function () {
        return _storyboard.mainStory.debug('webpack', 'Finished bundling!');
      });
    }, new _webpack2.default.DefinePlugin({
      'process.env.NODE_ENV': JSON.stringify(fProduction ? 'production' : 'development')
    })];
    if (fProduction) {
      ret.push(new _webpack2.default.optimize.UglifyJsPlugin({
        compress: { warnings: false },
        sourceMap: false
      }));
    } else {
      ret.push(new _webpack2.default.HotModuleReplacementPlugin());
      ret.push(new _webpack2.default.NoErrorsPlugin());
    }
    return ret;
  }(),

  module: {
    loaders: [{
      test: /\.(js|jsx)$/,
      loader: 'babel',
      exclude: _path2.default.resolve(process.cwd(), 'node_modules')
    }, {
      test: /\.(otf|eot|svg|ttf|woff|woff2)(\?v=[0-9]\.[0-9]\.[0-9])?$/,
      loader: 'file'
    }, {
      test: /\.css$/,
      loader: _styleLoader('css')
    }, {
      test: /\.sass$/,
      loader: _styleLoader('css!sass?indentedSyntax')
    }, {
      test: /\.png$/,
      loader: 'file'
    }, {
      test: /\.json$/,
      loader: 'json'
    }]
  }
};